# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..cog import variants as cogvariants
import typing
from ..models import dashboard
import enum
from ..cog import runtime as cogruntime


class AzureMonitorQuery(cogvariants.Dataquery):
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # Azure subscription containing the resource(s) to be queried.
    subscription: typing.Optional[str]
    # Subscriptions to be queried via Azure Resource Graph.
    subscriptions: typing.Optional[list[str]]
    # Azure Monitor Metrics sub-query properties.
    azure_monitor: typing.Optional['AzureMetricQuery']
    # Azure Monitor Logs sub-query properties.
    azure_log_analytics: typing.Optional['AzureLogsQuery']
    # Azure Resource Graph sub-query properties.
    azure_resource_graph: typing.Optional['AzureResourceGraphQuery']
    # Application Insights Traces sub-query properties.
    azure_traces: typing.Optional['AzureTracesQuery']
    # @deprecated Legacy template variable support.
    grafana_template_variable_fn: typing.Optional['GrafanaTemplateVariableQuery']
    # Template variables params. These exist for backwards compatiblity with legacy template variables.
    resource_group: typing.Optional[str]
    namespace: typing.Optional[str]
    resource: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]
    # Azure Monitor query type.
    # queryType: #AzureQueryType
    region: typing.Optional[str]

    def __init__(self, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, subscription: typing.Optional[str] = None, subscriptions: typing.Optional[list[str]] = None, azure_monitor: typing.Optional['AzureMetricQuery'] = None, azure_log_analytics: typing.Optional['AzureLogsQuery'] = None, azure_resource_graph: typing.Optional['AzureResourceGraphQuery'] = None, azure_traces: typing.Optional['AzureTracesQuery'] = None, grafana_template_variable_fn: typing.Optional['GrafanaTemplateVariableQuery'] = None, resource_group: typing.Optional[str] = None, namespace: typing.Optional[str] = None, resource: typing.Optional[str] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, region: typing.Optional[str] = None):
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.subscription = subscription
        self.subscriptions = subscriptions
        self.azure_monitor = azure_monitor
        self.azure_log_analytics = azure_log_analytics
        self.azure_resource_graph = azure_resource_graph
        self.azure_traces = azure_traces
        self.grafana_template_variable_fn = grafana_template_variable_fn
        self.resource_group = resource_group
        self.namespace = namespace
        self.resource = resource
        self.datasource = datasource
        self.region = region

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "refId": self.ref_id,
        }
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.subscription is not None:
            payload["subscription"] = self.subscription
        if self.subscriptions is not None:
            payload["subscriptions"] = self.subscriptions
        if self.azure_monitor is not None:
            payload["azureMonitor"] = self.azure_monitor
        if self.azure_log_analytics is not None:
            payload["azureLogAnalytics"] = self.azure_log_analytics
        if self.azure_resource_graph is not None:
            payload["azureResourceGraph"] = self.azure_resource_graph
        if self.azure_traces is not None:
            payload["azureTraces"] = self.azure_traces
        if self.grafana_template_variable_fn is not None:
            payload["grafanaTemplateVariableFn"] = self.grafana_template_variable_fn
        if self.resource_group is not None:
            payload["resourceGroup"] = self.resource_group
        if self.namespace is not None:
            payload["namespace"] = self.namespace
        if self.resource is not None:
            payload["resource"] = self.resource
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.region is not None:
            payload["region"] = self.region
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]
        if "subscriptions" in data:
            args["subscriptions"] = data["subscriptions"]
        if "azureMonitor" in data:
            args["azure_monitor"] = AzureMetricQuery.from_json(data["azureMonitor"])
        if "azureLogAnalytics" in data:
            args["azure_log_analytics"] = AzureLogsQuery.from_json(data["azureLogAnalytics"])
        if "azureResourceGraph" in data:
            args["azure_resource_graph"] = AzureResourceGraphQuery.from_json(data["azureResourceGraph"])
        if "azureTraces" in data:
            args["azure_traces"] = AzureTracesQuery.from_json(data["azureTraces"])
        if "grafanaTemplateVariableFn" in data:
            decoding_map_grafanaTemplateVariableFn_ref_union: dict[str, typing.Union[typing.Type[AppInsightsGroupByQuery], typing.Type[AppInsightsMetricNameQuery], typing.Type[MetricDefinitionsQuery], typing.Type[MetricNamesQuery], typing.Type[MetricNamespaceQuery], typing.Type[ResourceGroupsQuery], typing.Type[ResourceNamesQuery], typing.Type[SubscriptionsQuery], typing.Type[UnknownQuery], typing.Type[WorkspacesQuery]]] = {"AppInsightsGroupByQuery": AppInsightsGroupByQuery, "AppInsightsMetricNameQuery": AppInsightsMetricNameQuery, "MetricDefinitionsQuery": MetricDefinitionsQuery, "MetricNamesQuery": MetricNamesQuery, "MetricNamespaceQuery": MetricNamespaceQuery, "ResourceGroupsQuery": ResourceGroupsQuery, "ResourceNamesQuery": ResourceNamesQuery, "SubscriptionsQuery": SubscriptionsQuery, "UnknownQuery": UnknownQuery, "WorkspacesQuery": WorkspacesQuery}
            args["grafana_template_variable_fn"] = decoding_map_grafanaTemplateVariableFn_ref_union[data["grafanaTemplateVariableFn"]["kind"]].from_json(data["grafanaTemplateVariableFn"])
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "namespace" in data:
            args["namespace"] = data["namespace"]
        if "resource" in data:
            args["resource"] = data["resource"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "region" in data:
            args["region"] = data["region"]        

        return cls(**args)


class AzureMetricQuery:
    # Array of resource URIs to be queried.
    resources: typing.Optional[list['AzureMonitorResource']]
    # metricNamespace is used as the resource type (or resource namespace).
    # It's usually equal to the target metric namespace. e.g. microsoft.storage/storageaccounts
    # Kept the name of the variable as metricNamespace to avoid backward incompatibility issues.
    metric_namespace: typing.Optional[str]
    # Used as the value for the metricNamespace property when it's different from the resource namespace.
    custom_namespace: typing.Optional[str]
    # The metric to query data for within the specified metricNamespace. e.g. UsedCapacity
    metric_name: typing.Optional[str]
    # The Azure region containing the resource(s).
    region: typing.Optional[str]
    # The granularity of data points to be queried. Defaults to auto.
    time_grain: typing.Optional[str]
    # The aggregation to be used within the query. Defaults to the primaryAggregationType defined by the metric.
    aggregation: typing.Optional[str]
    # Filters to reduce the set of data returned. Dimensions that can be filtered on are defined by the metric.
    dimension_filters: typing.Optional[list['AzureMetricDimension']]
    # Maximum number of records to return. Defaults to 10.
    top: typing.Optional[str]
    # Time grains that are supported by the metric.
    allowed_time_grains_ms: typing.Optional[list[int]]
    # Aliases can be set to modify the legend labels. e.g. {{ resourceGroup }}. See docs for more detail.
    alias: typing.Optional[str]
    # @deprecated
    time_grain_unit: typing.Optional[str]
    # @deprecated This property was migrated to dimensionFilters and should only be accessed in the migration
    dimension: typing.Optional[str]
    # @deprecated This property was migrated to dimensionFilters and should only be accessed in the migration
    dimension_filter: typing.Optional[str]
    # @deprecated Use metricNamespace instead
    metric_definition: typing.Optional[str]
    # @deprecated Use resourceGroup, resourceName and metricNamespace instead
    resource_uri: typing.Optional[str]
    # @deprecated Use resources instead
    resource_group: typing.Optional[str]
    # @deprecated Use resources instead
    resource_name: typing.Optional[str]

    def __init__(self, resources: typing.Optional[list['AzureMonitorResource']] = None, metric_namespace: typing.Optional[str] = None, custom_namespace: typing.Optional[str] = None, metric_name: typing.Optional[str] = None, region: typing.Optional[str] = None, time_grain: typing.Optional[str] = None, aggregation: typing.Optional[str] = None, dimension_filters: typing.Optional[list['AzureMetricDimension']] = None, top: typing.Optional[str] = None, allowed_time_grains_ms: typing.Optional[list[int]] = None, alias: typing.Optional[str] = None, time_grain_unit: typing.Optional[str] = None, dimension: typing.Optional[str] = None, dimension_filter: typing.Optional[str] = None, metric_definition: typing.Optional[str] = None, resource_uri: typing.Optional[str] = None, resource_group: typing.Optional[str] = None, resource_name: typing.Optional[str] = None):
        self.resources = resources
        self.metric_namespace = metric_namespace
        self.custom_namespace = custom_namespace
        self.metric_name = metric_name
        self.region = region
        self.time_grain = time_grain
        self.aggregation = aggregation
        self.dimension_filters = dimension_filters
        self.top = top
        self.allowed_time_grains_ms = allowed_time_grains_ms
        self.alias = alias
        self.time_grain_unit = time_grain_unit
        self.dimension = dimension
        self.dimension_filter = dimension_filter
        self.metric_definition = metric_definition
        self.resource_uri = resource_uri
        self.resource_group = resource_group
        self.resource_name = resource_name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.resources is not None:
            payload["resources"] = self.resources
        if self.metric_namespace is not None:
            payload["metricNamespace"] = self.metric_namespace
        if self.custom_namespace is not None:
            payload["customNamespace"] = self.custom_namespace
        if self.metric_name is not None:
            payload["metricName"] = self.metric_name
        if self.region is not None:
            payload["region"] = self.region
        if self.time_grain is not None:
            payload["timeGrain"] = self.time_grain
        if self.aggregation is not None:
            payload["aggregation"] = self.aggregation
        if self.dimension_filters is not None:
            payload["dimensionFilters"] = self.dimension_filters
        if self.top is not None:
            payload["top"] = self.top
        if self.allowed_time_grains_ms is not None:
            payload["allowedTimeGrainsMs"] = self.allowed_time_grains_ms
        if self.alias is not None:
            payload["alias"] = self.alias
        if self.time_grain_unit is not None:
            payload["timeGrainUnit"] = self.time_grain_unit
        if self.dimension is not None:
            payload["dimension"] = self.dimension
        if self.dimension_filter is not None:
            payload["dimensionFilter"] = self.dimension_filter
        if self.metric_definition is not None:
            payload["metricDefinition"] = self.metric_definition
        if self.resource_uri is not None:
            payload["resourceUri"] = self.resource_uri
        if self.resource_group is not None:
            payload["resourceGroup"] = self.resource_group
        if self.resource_name is not None:
            payload["resourceName"] = self.resource_name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "resources" in data:
            args["resources"] = [AzureMonitorResource.from_json(item) for item in data["resources"]]
        if "metricNamespace" in data:
            args["metric_namespace"] = data["metricNamespace"]
        if "customNamespace" in data:
            args["custom_namespace"] = data["customNamespace"]
        if "metricName" in data:
            args["metric_name"] = data["metricName"]
        if "region" in data:
            args["region"] = data["region"]
        if "timeGrain" in data:
            args["time_grain"] = data["timeGrain"]
        if "aggregation" in data:
            args["aggregation"] = data["aggregation"]
        if "dimensionFilters" in data:
            args["dimension_filters"] = [AzureMetricDimension.from_json(item) for item in data["dimensionFilters"]]
        if "top" in data:
            args["top"] = data["top"]
        if "allowedTimeGrainsMs" in data:
            args["allowed_time_grains_ms"] = data["allowedTimeGrainsMs"]
        if "alias" in data:
            args["alias"] = data["alias"]
        if "timeGrainUnit" in data:
            args["time_grain_unit"] = data["timeGrainUnit"]
        if "dimension" in data:
            args["dimension"] = data["dimension"]
        if "dimensionFilter" in data:
            args["dimension_filter"] = data["dimensionFilter"]
        if "metricDefinition" in data:
            args["metric_definition"] = data["metricDefinition"]
        if "resourceUri" in data:
            args["resource_uri"] = data["resourceUri"]
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "resourceName" in data:
            args["resource_name"] = data["resourceName"]        

        return cls(**args)


class AzureMonitorResource:
    subscription: typing.Optional[str]
    resource_group: typing.Optional[str]
    resource_name: typing.Optional[str]
    metric_namespace: typing.Optional[str]
    region: typing.Optional[str]

    def __init__(self, subscription: typing.Optional[str] = None, resource_group: typing.Optional[str] = None, resource_name: typing.Optional[str] = None, metric_namespace: typing.Optional[str] = None, region: typing.Optional[str] = None):
        self.subscription = subscription
        self.resource_group = resource_group
        self.resource_name = resource_name
        self.metric_namespace = metric_namespace
        self.region = region

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.subscription is not None:
            payload["subscription"] = self.subscription
        if self.resource_group is not None:
            payload["resourceGroup"] = self.resource_group
        if self.resource_name is not None:
            payload["resourceName"] = self.resource_name
        if self.metric_namespace is not None:
            payload["metricNamespace"] = self.metric_namespace
        if self.region is not None:
            payload["region"] = self.region
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "subscription" in data:
            args["subscription"] = data["subscription"]
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "resourceName" in data:
            args["resource_name"] = data["resourceName"]
        if "metricNamespace" in data:
            args["metric_namespace"] = data["metricNamespace"]
        if "region" in data:
            args["region"] = data["region"]        

        return cls(**args)


class AzureMetricDimension:
    # Name of Dimension to be filtered on.
    dimension: typing.Optional[str]
    # String denoting the filter operation. Supports 'eq' - equals,'ne' - not equals, 'sw' - starts with. Note that some dimensions may not support all operators.
    operator: typing.Optional[str]
    # Values to match with the filter.
    filters: typing.Optional[list[str]]
    # @deprecated filter is deprecated in favour of filters to support multiselect.
    filter_val: typing.Optional[str]

    def __init__(self, dimension: typing.Optional[str] = None, operator: typing.Optional[str] = None, filters: typing.Optional[list[str]] = None, filter_val: typing.Optional[str] = None):
        self.dimension = dimension
        self.operator = operator
        self.filters = filters
        self.filter_val = filter_val

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.dimension is not None:
            payload["dimension"] = self.dimension
        if self.operator is not None:
            payload["operator"] = self.operator
        if self.filters is not None:
            payload["filters"] = self.filters
        if self.filter_val is not None:
            payload["filter"] = self.filter_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "dimension" in data:
            args["dimension"] = data["dimension"]
        if "operator" in data:
            args["operator"] = data["operator"]
        if "filters" in data:
            args["filters"] = data["filters"]
        if "filter" in data:
            args["filter_val"] = data["filter"]        

        return cls(**args)


class AzureLogsQuery:
    """
    Azure Monitor Logs sub-query properties
    """

    # KQL query to be executed.
    query: typing.Optional[str]
    # Specifies the format results should be returned as.
    result_format: typing.Optional['ResultFormat']
    # Array of resource URIs to be queried.
    resources: typing.Optional[list[str]]
    # If set to true the dashboard time range will be used as a filter for the query. Otherwise the query time ranges will be used. Defaults to false.
    dashboard_time: typing.Optional[bool]
    # If dashboardTime is set to true this value dictates which column the time filter will be applied to. Defaults to the first tables timeSpan column, the first datetime column found, or TimeGenerated
    time_column: typing.Optional[str]
    # Workspace ID. This was removed in Grafana 8, but remains for backwards compat.
    workspace: typing.Optional[str]
    # @deprecated Use resources instead
    resource: typing.Optional[str]
    # @deprecated Use dashboardTime instead
    intersect_time: typing.Optional[bool]

    def __init__(self, query: typing.Optional[str] = None, result_format: typing.Optional['ResultFormat'] = None, resources: typing.Optional[list[str]] = None, dashboard_time: typing.Optional[bool] = None, time_column: typing.Optional[str] = None, workspace: typing.Optional[str] = None, resource: typing.Optional[str] = None, intersect_time: typing.Optional[bool] = None):
        self.query = query
        self.result_format = result_format
        self.resources = resources
        self.dashboard_time = dashboard_time
        self.time_column = time_column
        self.workspace = workspace
        self.resource = resource
        self.intersect_time = intersect_time

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.query is not None:
            payload["query"] = self.query
        if self.result_format is not None:
            payload["resultFormat"] = self.result_format
        if self.resources is not None:
            payload["resources"] = self.resources
        if self.dashboard_time is not None:
            payload["dashboardTime"] = self.dashboard_time
        if self.time_column is not None:
            payload["timeColumn"] = self.time_column
        if self.workspace is not None:
            payload["workspace"] = self.workspace
        if self.resource is not None:
            payload["resource"] = self.resource
        if self.intersect_time is not None:
            payload["intersectTime"] = self.intersect_time
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "query" in data:
            args["query"] = data["query"]
        if "resultFormat" in data:
            args["result_format"] = data["resultFormat"]
        if "resources" in data:
            args["resources"] = data["resources"]
        if "dashboardTime" in data:
            args["dashboard_time"] = data["dashboardTime"]
        if "timeColumn" in data:
            args["time_column"] = data["timeColumn"]
        if "workspace" in data:
            args["workspace"] = data["workspace"]
        if "resource" in data:
            args["resource"] = data["resource"]
        if "intersectTime" in data:
            args["intersect_time"] = data["intersectTime"]        

        return cls(**args)


class ResultFormat(enum.StrEnum):
    TABLE = "table"
    TIME_SERIES = "time_series"
    TRACE = "trace"
    LOGS = "logs"


class AzureResourceGraphQuery:
    # Azure Resource Graph KQL query to be executed.
    query: typing.Optional[str]
    # Specifies the format results should be returned as. Defaults to table.
    result_format: typing.Optional[str]

    def __init__(self, query: typing.Optional[str] = None, result_format: typing.Optional[str] = None):
        self.query = query
        self.result_format = result_format

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.query is not None:
            payload["query"] = self.query
        if self.result_format is not None:
            payload["resultFormat"] = self.result_format
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "query" in data:
            args["query"] = data["query"]
        if "resultFormat" in data:
            args["result_format"] = data["resultFormat"]        

        return cls(**args)


class AzureTracesQuery:
    """
    Application Insights Traces sub-query properties
    """

    # Specifies the format results should be returned as.
    result_format: typing.Optional['ResultFormat']
    # Array of resource URIs to be queried.
    resources: typing.Optional[list[str]]
    # Operation ID. Used only for Traces queries.
    operation_id: typing.Optional[str]
    # Types of events to filter by.
    trace_types: typing.Optional[list[str]]
    # Filters for property values.
    filters: typing.Optional[list['AzureTracesFilter']]
    # KQL query to be executed.
    query: typing.Optional[str]

    def __init__(self, result_format: typing.Optional['ResultFormat'] = None, resources: typing.Optional[list[str]] = None, operation_id: typing.Optional[str] = None, trace_types: typing.Optional[list[str]] = None, filters: typing.Optional[list['AzureTracesFilter']] = None, query: typing.Optional[str] = None):
        self.result_format = result_format
        self.resources = resources
        self.operation_id = operation_id
        self.trace_types = trace_types
        self.filters = filters
        self.query = query

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.result_format is not None:
            payload["resultFormat"] = self.result_format
        if self.resources is not None:
            payload["resources"] = self.resources
        if self.operation_id is not None:
            payload["operationId"] = self.operation_id
        if self.trace_types is not None:
            payload["traceTypes"] = self.trace_types
        if self.filters is not None:
            payload["filters"] = self.filters
        if self.query is not None:
            payload["query"] = self.query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "resultFormat" in data:
            args["result_format"] = data["resultFormat"]
        if "resources" in data:
            args["resources"] = data["resources"]
        if "operationId" in data:
            args["operation_id"] = data["operationId"]
        if "traceTypes" in data:
            args["trace_types"] = data["traceTypes"]
        if "filters" in data:
            args["filters"] = [AzureTracesFilter.from_json(item) for item in data["filters"]]
        if "query" in data:
            args["query"] = data["query"]        

        return cls(**args)


class AzureTracesFilter:
    # Property name, auto-populated based on available traces.
    property_val: str
    # Comparison operator to use. Either equals or not equals.
    operation: str
    # Values to filter by.
    filters: list[str]

    def __init__(self, property_val: str = "", operation: str = "", filters: typing.Optional[list[str]] = None):
        self.property_val = property_val
        self.operation = operation
        self.filters = filters if filters is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "property": self.property_val,
            "operation": self.operation,
            "filters": self.filters,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "property" in data:
            args["property_val"] = data["property"]
        if "operation" in data:
            args["operation"] = data["operation"]
        if "filters" in data:
            args["filters"] = data["filters"]        

        return cls(**args)


GrafanaTemplateVariableQuery: typing.TypeAlias = typing.Union['AppInsightsMetricNameQuery', 'AppInsightsGroupByQuery', 'SubscriptionsQuery', 'ResourceGroupsQuery', 'ResourceNamesQuery', 'MetricNamespaceQuery', 'MetricDefinitionsQuery', 'MetricNamesQuery', 'WorkspacesQuery', 'UnknownQuery']


class AppInsightsMetricNameQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["AppInsightsMetricNameQuery"]

    def __init__(self, raw_query: typing.Optional[str] = None):
        self.raw_query = raw_query
        self.kind = "AppInsightsMetricNameQuery"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]        

        return cls(**args)


class AppInsightsGroupByQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["AppInsightsGroupByQuery"]
    metric_name: str

    def __init__(self, raw_query: typing.Optional[str] = None, metric_name: str = ""):
        self.raw_query = raw_query
        self.kind = "AppInsightsGroupByQuery"
        self.metric_name = metric_name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "metricName": self.metric_name,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "metricName" in data:
            args["metric_name"] = data["metricName"]        

        return cls(**args)


class SubscriptionsQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["SubscriptionsQuery"]

    def __init__(self, raw_query: typing.Optional[str] = None):
        self.raw_query = raw_query
        self.kind = "SubscriptionsQuery"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]        

        return cls(**args)


class ResourceGroupsQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["ResourceGroupsQuery"]
    subscription: str

    def __init__(self, raw_query: typing.Optional[str] = None, subscription: str = ""):
        self.raw_query = raw_query
        self.kind = "ResourceGroupsQuery"
        self.subscription = subscription

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "subscription": self.subscription,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]        

        return cls(**args)


class ResourceNamesQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["ResourceNamesQuery"]
    subscription: str
    resource_group: str
    metric_namespace: str

    def __init__(self, raw_query: typing.Optional[str] = None, subscription: str = "", resource_group: str = "", metric_namespace: str = ""):
        self.raw_query = raw_query
        self.kind = "ResourceNamesQuery"
        self.subscription = subscription
        self.resource_group = resource_group
        self.metric_namespace = metric_namespace

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "subscription": self.subscription,
            "resourceGroup": self.resource_group,
            "metricNamespace": self.metric_namespace,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "metricNamespace" in data:
            args["metric_namespace"] = data["metricNamespace"]        

        return cls(**args)


class MetricNamespaceQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["MetricNamespaceQuery"]
    subscription: str
    resource_group: str
    metric_namespace: typing.Optional[str]
    resource_name: typing.Optional[str]

    def __init__(self, raw_query: typing.Optional[str] = None, subscription: str = "", resource_group: str = "", metric_namespace: typing.Optional[str] = None, resource_name: typing.Optional[str] = None):
        self.raw_query = raw_query
        self.kind = "MetricNamespaceQuery"
        self.subscription = subscription
        self.resource_group = resource_group
        self.metric_namespace = metric_namespace
        self.resource_name = resource_name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "subscription": self.subscription,
            "resourceGroup": self.resource_group,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        if self.metric_namespace is not None:
            payload["metricNamespace"] = self.metric_namespace
        if self.resource_name is not None:
            payload["resourceName"] = self.resource_name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "metricNamespace" in data:
            args["metric_namespace"] = data["metricNamespace"]
        if "resourceName" in data:
            args["resource_name"] = data["resourceName"]        

        return cls(**args)


class MetricDefinitionsQuery:
    """
    @deprecated Use MetricNamespaceQuery instead
    """

    raw_query: typing.Optional[str]
    kind: typing.Literal["MetricDefinitionsQuery"]
    subscription: str
    resource_group: str
    metric_namespace: typing.Optional[str]
    resource_name: typing.Optional[str]

    def __init__(self, raw_query: typing.Optional[str] = None, subscription: str = "", resource_group: str = "", metric_namespace: typing.Optional[str] = None, resource_name: typing.Optional[str] = None):
        self.raw_query = raw_query
        self.kind = "MetricDefinitionsQuery"
        self.subscription = subscription
        self.resource_group = resource_group
        self.metric_namespace = metric_namespace
        self.resource_name = resource_name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "subscription": self.subscription,
            "resourceGroup": self.resource_group,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        if self.metric_namespace is not None:
            payload["metricNamespace"] = self.metric_namespace
        if self.resource_name is not None:
            payload["resourceName"] = self.resource_name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "metricNamespace" in data:
            args["metric_namespace"] = data["metricNamespace"]
        if "resourceName" in data:
            args["resource_name"] = data["resourceName"]        

        return cls(**args)


class MetricNamesQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["MetricNamesQuery"]
    subscription: str
    resource_group: str
    resource_name: str
    metric_namespace: str

    def __init__(self, raw_query: typing.Optional[str] = None, subscription: str = "", resource_group: str = "", resource_name: str = "", metric_namespace: str = ""):
        self.raw_query = raw_query
        self.kind = "MetricNamesQuery"
        self.subscription = subscription
        self.resource_group = resource_group
        self.resource_name = resource_name
        self.metric_namespace = metric_namespace

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "subscription": self.subscription,
            "resourceGroup": self.resource_group,
            "resourceName": self.resource_name,
            "metricNamespace": self.metric_namespace,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]
        if "resourceGroup" in data:
            args["resource_group"] = data["resourceGroup"]
        if "resourceName" in data:
            args["resource_name"] = data["resourceName"]
        if "metricNamespace" in data:
            args["metric_namespace"] = data["metricNamespace"]        

        return cls(**args)


class WorkspacesQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["WorkspacesQuery"]
    subscription: str

    def __init__(self, raw_query: typing.Optional[str] = None, subscription: str = ""):
        self.raw_query = raw_query
        self.kind = "WorkspacesQuery"
        self.subscription = subscription

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "subscription": self.subscription,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "subscription" in data:
            args["subscription"] = data["subscription"]        

        return cls(**args)


class UnknownQuery:
    raw_query: typing.Optional[str]
    kind: typing.Literal["UnknownQuery"]

    def __init__(self, raw_query: typing.Optional[str] = None):
        self.raw_query = raw_query
        self.kind = "UnknownQuery"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]        

        return cls(**args)


class AzureQueryType(enum.StrEnum):
    """
    Defines the supported queryTypes. GrafanaTemplateVariableFn is deprecated
    """

    AZURE_MONITOR = "Azure Monitor"
    LOG_ANALYTICS = "Azure Log Analytics"
    AZURE_RESOURCE_GRAPH = "Azure Resource Graph"
    AZURE_TRACES = "Azure Traces"
    SUBSCRIPTIONS_QUERY = "Azure Subscriptions"
    RESOURCE_GROUPS_QUERY = "Azure Resource Groups"
    NAMESPACES_QUERY = "Azure Namespaces"
    RESOURCE_NAMES_QUERY = "Azure Resource Names"
    METRIC_NAMES_QUERY = "Azure Metric Names"
    WORKSPACES_QUERY = "Azure Workspaces"
    LOCATIONS_QUERY = "Azure Regions"
    GRAFANA_TEMPLATE_VARIABLE_FN = "Grafana Template Variable Function"


class GrafanaTemplateVariableQueryType(enum.StrEnum):
    APP_INSIGHTS_METRIC_NAME_QUERY = "AppInsightsMetricNameQuery"
    APP_INSIGHTS_GROUP_BY_QUERY = "AppInsightsGroupByQuery"
    SUBSCRIPTIONS_QUERY = "SubscriptionsQuery"
    RESOURCE_GROUPS_QUERY = "ResourceGroupsQuery"
    RESOURCE_NAMES_QUERY = "ResourceNamesQuery"
    METRIC_NAMESPACE_QUERY = "MetricNamespaceQuery"
    METRIC_NAMES_QUERY = "MetricNamesQuery"
    WORKSPACES_QUERY = "WorkspacesQuery"
    UNKNOWN_QUERY = "UnknownQuery"


class BaseGrafanaTemplateVariableQuery:
    raw_query: typing.Optional[str]

    def __init__(self, raw_query: typing.Optional[str] = None):
        self.raw_query = raw_query

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="grafana-azure-monitor-datasource",
        from_json_hook=AzureMonitorQuery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import runtime as cogruntime


class Options:
    selected_series: int

    def __init__(self, selected_series: int = 0):
        self.selected_series = selected_series

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "selectedSeries": self.selected_series,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "selectedSeries" in data:
            args["selected_series"] = data["selectedSeries"]        

        return cls(**args)


def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="datagrid",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


NoUnit: typing.Literal["none"] = "none"


Number: typing.Literal["none"] = "none"


String: typing.Literal["string"] = "string"


Short: typing.Literal["short"] = "short"


# SI short
SiShort: typing.Literal["sishort"] = "sishort"


# Percent (0-100)
Percent: typing.Literal["percent"] = "percent"


# Percent (0.0-1.0)
PercentUnit: typing.Literal["percentunit"] = "percentunit"


# Humidity (%H)
Humidity: typing.Literal["humidity"] = "humidity"


# Decibel (dB)
Decibel: typing.Literal["dB"] = "dB"


# Candela (cd)
Candela: typing.Literal["candela"] = "candela"


# Hexadecimal (0x)
HexadecimalOx: typing.Literal["hex0x"] = "hex0x"


Hexadecimal: typing.Literal["hex"] = "hex"


ScientificNotation: typing.Literal["sci"] = "sci"


LocaleFormat: typing.Literal["locale"] = "locale"


# Pixels (px)
Pixels: typing.Literal["pixel"] = "pixel"


# Acceleration: Meters/sec² (m/sec²)
MetersSecSquared: typing.Literal["accMS2"] = "accMS2"


# Acceleration: Feet/sec² (f/sec²)
FeetSecSquared: typing.Literal["accFS2"] = "accFS2"


# G unit (g)
GUnit: typing.Literal["accG"] = "accG"


# Angle: Degrees (°)
Degrees: typing.Literal["degree"] = "degree"


# Angle: Radians
Radians: typing.Literal["radian"] = "radian"


# Angle: Gradian
Gradian: typing.Literal["grad"] = "grad"


# Angle: Arc Minutes
ArcMinutes: typing.Literal["arcmin"] = "arcmin"


# Angle: Arc Seconds
ArcSeconds: typing.Literal["arcsec"] = "arcsec"


# Area: Square Meters (m²)
SquareMeters: typing.Literal["areaM2"] = "areaM2"


# Area: Square Feet (ft²)
SquareFeet: typing.Literal["areaF2"] = "areaF2"


# Area: Square Miles (mi²)
SquareMiles: typing.Literal["areaMI2"] = "areaMI2"


# Area: Acres (ac)
Acres: typing.Literal["acres"] = "acres"


# Area: Hectares (ha)
Hectares: typing.Literal["hectares"] = "hectares"


# Computation: FLOP/s
FlopsPerSecond: typing.Literal["flops"] = "flops"


# Computation: MFLOP/s
MegaFlopsPerSecond: typing.Literal["mflops"] = "mflops"


# Computation: GFLOP/s
GigaFlopsPerSecond: typing.Literal["gflops"] = "gflops"


# Computation: TFLOP/s
TeraFlopsPerSecond: typing.Literal["tflops"] = "tflops"


# Computation: PFLOP/s
PetaFlopsPerSecond: typing.Literal["pflops"] = "pflops"


# Computation: EFLOP/s
ExaFlopsPerSecond: typing.Literal["eflops"] = "eflops"


# Computation: ZFLOP/s
ZettaFlopsPerSecond: typing.Literal["zflops"] = "zflops"


# Computation: YFLOP/s
YottaFlopsPerSecond: typing.Literal["yflops"] = "yflops"


# Concentration: parts-per-million (ppm)
PartsPerMillion: typing.Literal["ppm"] = "ppm"


# Concentration: parts-per-billion (ppb)
PartsPerBillion: typing.Literal["conppb"] = "conppb"


# Concentration: nanogram per cubic meter (ng/m³)
NanogramPerCubicMeter: typing.Literal["conngm3"] = "conngm3"


# Concentration: nanogram per normal cubic meter (ng/Nm³)
NanogramPerNormalCubicMeter: typing.Literal["conngNm3"] = "conngNm3"


# Concentration: microgram per cubic meter (μg/m³)
MicrogramPerCubicMeter: typing.Literal["conμgm3"] = "conμgm3"


# Concentration: microgram per normal cubic meter (μg/Nm³)
MicrogramPerNormalCubicMeter: typing.Literal["conμgNm3"] = "conμgNm3"


# Concentration: milligram per cubic meter (mg/m³)
MilligramPerCubicMeter: typing.Literal["conmgm3"] = "conmgm3"


# Concentration: milligram per normal cubic meter (mg/Nm³)
MilligramPerNormalCubicMeter: typing.Literal["conmgNm3"] = "conmgNm3"


# Concentration: gram per cubic meter (g/m³)
GramPerCubicMeter: typing.Literal["congm3"] = "congm3"


# Concentration: gram per normal cubic meter (g/Nm³)
GramPerNormalCubicMeter: typing.Literal["congNm3"] = "congNm3"


# Concentration: milligrams per decilitre (mg/dL)
MilligramsPerDecilitre: typing.Literal["conmgdL"] = "conmgdL"


# Concentration: millimoles per litre (mmol/L)
MillimolesPerLiter: typing.Literal["conmmolL"] = "conmmolL"


# Currency: Dollars ($)
Dollars: typing.Literal["currencyUSD"] = "currencyUSD"


# Currency: Pounds (£)
Pounds: typing.Literal["currencyGBP"] = "currencyGBP"


# Currency: Euro (€)
Euro: typing.Literal["currencyEUR"] = "currencyEUR"


# Currency: Yen (¥)
Yen: typing.Literal["currencyJPY"] = "currencyJPY"


# Currency: Rubles (₽)
Rubles: typing.Literal["currencyRUB"] = "currencyRUB"


# Currency: Hryvnias (₴)
Hryvnias: typing.Literal["currencyUAH"] = "currencyUAH"


# Currency: Real (R$)
Real: typing.Literal["currencyBRL"] = "currencyBRL"


# Currency: Danish Krone (kr)
DanishKrone: typing.Literal["currencyDKK"] = "currencyDKK"


# Currency: Icelandic Króna (kr)
IcelandicKrona: typing.Literal["currencyISK"] = "currencyISK"


# Currency: Norwegian Krone (kr)
NorwegianKrone: typing.Literal["currencyNOK"] = "currencyNOK"


# Currency: Swedish Krona (kr)
SwedishKrona: typing.Literal["currencySEK"] = "currencySEK"


# Currency: Czech koruna (czk)
CzechKoruna: typing.Literal["currencyCZK"] = "currencyCZK"


# Currency: Swiss franc (CHF)
SwissFranc: typing.Literal["currencyCHF"] = "currencyCHF"


# Currency: Polish Złoty (PLN)
PolishZloty: typing.Literal["currencyPLN"] = "currencyPLN"


# Currency: Bitcoin (฿)
Bictoin: typing.Literal["currencyBTC"] = "currencyBTC"


# Currency: Milli Bitcoin (฿)
MilliBitcoin: typing.Literal["currencymBTC"] = "currencymBTC"


# Currency: Micro Bitcoin (฿)
MicroBitcoin: typing.Literal["currencyμBTC"] = "currencyμBTC"


# Currency: South African Rand (R)
SouthAfricanRand: typing.Literal["currencyZAR"] = "currencyZAR"


# Currency: Indian Rupee (₹)
IndianRupee: typing.Literal["currencyINR"] = "currencyINR"


# Currency: South Korean Won (₩)
SouthKoreanWon: typing.Literal["currencyKRW"] = "currencyKRW"


# Currency: Indonesian Rupiah (Rp)
IndonesianRupiah: typing.Literal["currencyIDR"] = "currencyIDR"


# Currency: Philippine Peso (PHP)
PhilippinePeso: typing.Literal["currencyPHP"] = "currencyPHP"


# Currency: Vietnamese Dong (VND)
VietnameseDong: typing.Literal["currencyVND"] = "currencyVND"


# Currency: Turkish Lira (₺)
TurkishLira: typing.Literal["currencyTRY"] = "currencyTRY"


# Currency: Malaysian Ringgit (RM)
MalaysianRinggit: typing.Literal["currencyMYR"] = "currencyMYR"


# Currency: CFP franc (XPF)
CFPFranc: typing.Literal["currencyXPF"] = "currencyXPF"


# Currency: Bulgarian Lev (BGN)
BulgarianLev: typing.Literal["currencyBGN"] = "currencyBGN"


# Currency: Guaraní (₲)
Guarani: typing.Literal["currencyPYG"] = "currencyPYG"


# Currency: Uruguay Peso (UYU)
UruguayPeso: typing.Literal["currencyUYU"] = "currencyUYU"


# Currency: Israeli New Shekels (₪)
IsraeliNewShekels: typing.Literal["currencyILS"] = "currencyILS"


# Data: bytes(IEC)
BytesIEC: typing.Literal["bytes"] = "bytes"


# Data: bytes(SI)
BytesSI: typing.Literal["decbytes"] = "decbytes"


# Data: bits(IEC)
BitsIEC: typing.Literal["bits"] = "bits"


# Data: bits(SI)
BitsSI: typing.Literal["bydecbitstes"] = "bydecbitstes"


# Data: kibibytes
Kibibytes: typing.Literal["kbytes"] = "kbytes"


# Data: kilobytes
Kilobytes: typing.Literal["deckbytes"] = "deckbytes"


# Data: mebibytes
Mebibytes: typing.Literal["mbytes"] = "mbytes"


# Data: megabytes
Megabytes: typing.Literal["decmbytes"] = "decmbytes"


# Data: gibibytes
Gibibytes: typing.Literal["gbytes"] = "gbytes"


# Data: gigabytes
Gigabytes: typing.Literal["decgbytes"] = "decgbytes"


# Data: tebibytes
Tebibytes: typing.Literal["tbytes"] = "tbytes"


# Data: terabytes
Terabytes: typing.Literal["dectbytes"] = "dectbytes"


# Data: pebibytes
Pebibytes: typing.Literal["pbytes"] = "pbytes"


# Data: petabytes
Petabytes: typing.Literal["decpbytes"] = "decpbytes"


# Data rate: packets/sec
PacketsPerSecond: typing.Literal["pps"] = "pps"


# Data rate: bytes/sec(IEC)
BytesPerSecondIEC: typing.Literal["binBps"] = "binBps"


# Data rate: bytes/sec(SI)
BytesPerSecondSI: typing.Literal["Bps"] = "Bps"


# Data rate: bits/sec(IEC)
BitsPerSecondIEC: typing.Literal["binbps"] = "binbps"


# Data rate: bits/sec(SI)
BitsPerSecondSI: typing.Literal["bps"] = "bps"


# Data rate: kibibytes/sec
KibibytesPerSecond: typing.Literal["KiBs"] = "KiBs"


# Data rate: kibibits/sec
KibibitsPerSecond: typing.Literal["Kibits"] = "Kibits"


# Data rate: kilobytes/sec
KilobytesPerSecond: typing.Literal["KBs"] = "KBs"


# Data rate: kilobits/sec
KilobitsPerSecond: typing.Literal["Kbits"] = "Kbits"


# Data rate: mebibytes/sec
MebibytesPerSecond: typing.Literal["MiBs"] = "MiBs"


# Data rate: mebibits/sec
MebibitsPerSecond: typing.Literal["Mibits"] = "Mibits"


# Data rate: megabytes/sec
MegabytesPerSecond: typing.Literal["MBs"] = "MBs"


# Data rate: megabits/sec
MegabitsPerSecond: typing.Literal["Mbits"] = "Mbits"


# Data rate: gibibytes/sec
GibibytesPerSecond: typing.Literal["GiBs"] = "GiBs"


# Data rate: gibibits/sec
GibibitsPerSecond: typing.Literal["Gibits"] = "Gibits"


# Data rate: gigabytes/sec
GigabytesPerSecond: typing.Literal["GBs"] = "GBs"


# Data rate: gigabits/sec
GigabitsPerSecond: typing.Literal["Gbits"] = "Gbits"


# Data rate: tebibytes/sec
TebibytesPerSecond: typing.Literal["TiBs"] = "TiBs"


# Data rate: tebibits/sec
TebibitsPerSecond: typing.Literal["Tibits"] = "Tibits"


# Data rate: terabytes/sec
TerabytesPerSecond: typing.Literal["TBs"] = "TBs"


# Data rate: terabits/sec
TerabitsPerSecond: typing.Literal["Tbits"] = "Tbits"


# Data rate: pebibytes/sec
PebibytesPerSecond: typing.Literal["PiBs"] = "PiBs"


# Data rate: pebibits/sec
PebibitsPerSecond: typing.Literal["Pibits"] = "Pibits"


# Data rate: petabytes/sec
PetabytesPerSecond: typing.Literal["PBs"] = "PBs"


# Data rate: petabits/sec
PetabitsPerSecond: typing.Literal["Pbits"] = "Pbits"


# Date & time: Datetime ISO
DatetimeISO: typing.Literal["dateTimeAsIso"] = "dateTimeAsIso"


# Date & time: Datetime ISO (No date if today)
DatetimeISONotToday: typing.Literal["dateTimeAsIsoNoDateIfToday"] = "dateTimeAsIsoNoDateIfToday"


# Date & time: Datetime US
DatetimeUS: typing.Literal["dateTimeAsUS"] = "dateTimeAsUS"


# Date & time: Datetime US (No date if today)
DatetimeUSNotToday: typing.Literal["dateTimeAsUSNoDateIfToday"] = "dateTimeAsUSNoDateIfToday"


# Date & time: Datetime local
DatetimeLocal: typing.Literal["dateTimeAsLocal"] = "dateTimeAsLocal"


# Date & time: Datetime local (No date if today)
DatetimeLocalNotToday: typing.Literal["dateTimeAsLocalNoDateIfToday"] = "dateTimeAsLocalNoDateIfToday"


# Date & time: Datetime default
DatetimeDefault: typing.Literal["dateTimeAsSystem"] = "dateTimeAsSystem"


# Date & time: From Now
DateTimeFromNow: typing.Literal["dateTimeFromNow"] = "dateTimeFromNow"


# Energy: Watt (W)
Watt: typing.Literal["watt"] = "watt"


# Energy: Kilowatt (kW)
KiloWatt: typing.Literal["kwatt"] = "kwatt"


# Energy: Megawatt (MW)
MegaWatt: typing.Literal["megwatt"] = "megwatt"


# Energy: Gigawatt (GW)
GigaWatt: typing.Literal["gwatt"] = "gwatt"


# Energy: Milliwatt (mW)
MilliWatt: typing.Literal["mwatt"] = "mwatt"


# Energy: Watt per square meter (W/m²)
WattPerSquareMeter: typing.Literal["Wm2"] = "Wm2"


# Energy: Volt-Ampere (VA)
VoltAmpere: typing.Literal["voltamp"] = "voltamp"


# Energy: Kilovolt-Ampere (kVA)
KiloVoltAmpere: typing.Literal["kvoltamp"] = "kvoltamp"


# Energy: Volt-Ampere reactive (VAr)
VoltAmpereReactive: typing.Literal["voltampreact"] = "voltampreact"


# Energy: Kilovolt-Ampere reactive (kVAr)
KiloVoltAmpereReactive: typing.Literal["kvoltampreact"] = "kvoltampreact"


# Energy: Watt-hour (Wh)
WattHour: typing.Literal["watth"] = "watth"


# Energy: Watt-hour per Kilogram (Wh/kg)
WattHourPerKilogram: typing.Literal["watthperkg"] = "watthperkg"


# Energy: Kilowatt-hour (kWh)
KiloWattHour: typing.Literal["kwatth"] = "kwatth"


# Energy: Kilowatt-min (kWm)
KiloWattMinute: typing.Literal["kwattm"] = "kwattm"


# Energy: Megawatt-hour (MWh)
MegaWattHour: typing.Literal["mwatth"] = "mwatth"


# Energy: Ampere-hour (Ah)
AmpereHour: typing.Literal["amph"] = "amph"


# Energy: Kiloampere-hour (kAh)
KiloAmpereHour: typing.Literal["kamph"] = "kamph"


# Energy: Milliampere-hour (mAh)
MilliAmpereHour: typing.Literal["mamph"] = "mamph"


# Energy: Joule (J)
Joule: typing.Literal["joule"] = "joule"


# Energy: Electron volt (eV)
ElectronVolt: typing.Literal["ev"] = "ev"


# Energy: Ampere (A)
Ampere: typing.Literal["amp"] = "amp"


# Energy: Kiloampere (kA)
KiloAmpere: typing.Literal["kamp"] = "kamp"


# Energy: Milliampere (mA)
MilliAmpere: typing.Literal["mamp"] = "mamp"


# Energy: Volt (V)
Volt: typing.Literal["volt"] = "volt"


# Energy: Kilovolt (kV)
KiloVolt: typing.Literal["kvolt"] = "kvolt"


# Energy: Millivolt (mV)
MilliVolt: typing.Literal["mvolt"] = "mvolt"


# Energy: Decibel-milliwatt (dBm)
DecibelMilliWatt: typing.Literal["dBm"] = "dBm"


# Energy: Milliohm (mΩ)
MilliOhm: typing.Literal["mohm"] = "mohm"


# Energy: Ohm (Ω)
Ohm: typing.Literal["ohm"] = "ohm"


# Energy: Kiloohm (kΩ)
KiloOhm: typing.Literal["kohm"] = "kohm"


# Energy: Megaohm (MΩ)
MegaOhm: typing.Literal["Mohm"] = "Mohm"


# Energy: Farad (F)
Farad: typing.Literal["farad"] = "farad"


# Energy: Microfarad (µF)
MicroFarad: typing.Literal["watt"] = "watt"


# Energy: Nanofarad (nF)
NanoFarad: typing.Literal["nfarad"] = "nfarad"


# Energy: Picofarad (pF)
PicoFarad: typing.Literal["pfarad"] = "pfarad"


# Energy: Femtofarad (fF)
FemtoFarad: typing.Literal["ffarad"] = "ffarad"


# Energy: Henry (H)
Henry: typing.Literal["henry"] = "henry"


# Energy: Millihenry (mH)
MilliHenry: typing.Literal["mhenry"] = "mhenry"


# Energy: Microhenry (µH)
MicroHenry: typing.Literal["µhenry"] = "µhenry"


# Energy: Lumens (Lm)
Lumens: typing.Literal["lumens"] = "lumens"


# Flow: Gallons/min (gpm)
GallonsPerMinute: typing.Literal["flowgpm"] = "flowgpm"


# Flow: Cubic meters/sec (cms)
CubicMetersPerSecond: typing.Literal["flowcms"] = "flowcms"


# Flow: Cubic feet/sec (cfs)
CubicFeetPerSecond: typing.Literal["flowcms"] = "flowcms"


# Flow: Cubic feet/min (cfm)
CubicFeetPerMinute: typing.Literal["flowcms"] = "flowcms"


# Flow: Litre/hour
LitrePerHour: typing.Literal["litreh"] = "litreh"


# Flow: Litre/min (L/min)
LitrePerMinute: typing.Literal["flowlpm"] = "flowlpm"


# Flow: milliLitre/min (mL/min)
MilliLitrePerMinute: typing.Literal["flowmlpm"] = "flowmlpm"


# Flow: Lux (lx)
Lux: typing.Literal["lux"] = "lux"


# Force: Newton-meters (Nm)
NewtonMeters: typing.Literal["forceNm"] = "forceNm"


# Force: Kilonewton-meters (kNm)
KiloNewtonMeters: typing.Literal["forcekNm"] = "forcekNm"


# Force: Newtons (N)
Newtons: typing.Literal["forceN"] = "forceN"


# Force: Kilonewtons (kN)
KiloNewtons: typing.Literal["forcekN"] = "forcekN"


# Hash rate: hashes/sec
HashesPerSecond: typing.Literal["Hs"] = "Hs"


# Hash rate: kilohashes/sec
KiloHashesPerSecond: typing.Literal["KHs"] = "KHs"


# Hash rate: megahashes/sec
MegaHashesPerSecond: typing.Literal["MHs"] = "MHs"


# Hash rate: gigahashes/sec
GigaHashesPerSecond: typing.Literal["GHs"] = "GHs"


# Hash rate: terahashes/sec
TeraHashesPerSecond: typing.Literal["THs"] = "THs"


# Hash rate: petahashes/sec
PetaHashesPerSecond: typing.Literal["PHs"] = "PHs"


# Hash rate: exahashes/sec
ExaHashesPerSecond: typing.Literal["EHs"] = "EHs"


# Mass: milligram (mg)
Milligram: typing.Literal["massmg"] = "massmg"


# Mass: gram (g)
Gram: typing.Literal["massg"] = "massg"


# Mass: pound (lb)
Pound: typing.Literal["masslb"] = "masslb"


# Mass: kilogram (kg)
Kilogram: typing.Literal["masskg"] = "masskg"


# Mass: metric ton (t)
MetricTon: typing.Literal["masst"] = "masst"


# Length: millimeter (mm)
Millimeter: typing.Literal["lengthmm"] = "lengthmm"


# Length: inch (in)
Inch: typing.Literal["lengthin"] = "lengthin"


# Length: feet (ft)
Feet: typing.Literal["lengthft"] = "lengthft"


# Length: meter (m)
Meter: typing.Literal["lengthm"] = "lengthm"


# Length: kilometer (km)
Kilometer: typing.Literal["lengthkm"] = "lengthkm"


# Length: mile (mi)
Mile: typing.Literal["lengthmi"] = "lengthmi"


# Pressure: Millibars
Millibars: typing.Literal["pressurembar"] = "pressurembar"


# Pressure: Bars
Bars: typing.Literal["pressurebar"] = "pressurebar"


# Pressure: Kilobars
Kilobars: typing.Literal["pressurekbar"] = "pressurekbar"


# Pressure: Pascals
Pascals: typing.Literal["pressurepa"] = "pressurepa"


# Pressure: Hectopascals
Hectopascals: typing.Literal["pressurehpa"] = "pressurehpa"


# Pressure: Kilopascals
Kilopascals: typing.Literal["pressurekpa"] = "pressurekpa"


# Pressure: Inches of mercury
InchesOfMercury: typing.Literal["pressurehg"] = "pressurehg"


# Pressure: PSI
PSI: typing.Literal["pressurepsi"] = "pressurepsi"


# Radiation: Becquerel (Bq)
Becquerel: typing.Literal["radbq"] = "radbq"


# Radiation: curie (Ci)
Curie: typing.Literal["radci"] = "radci"


# Radiation: Gray (Gy)
Gray: typing.Literal["radgy"] = "radgy"


# Radiation: rad
Rad: typing.Literal["radrad"] = "radrad"


# Radiation: Sievert (Sv)
Sievert: typing.Literal["radsv"] = "radsv"


# Radiation: milliSievert (mSv)
Millisievert: typing.Literal["radmsv"] = "radmsv"


# Radiation: microSievert (µSv)
Microsievert: typing.Literal["radusv"] = "radusv"


# Radiation: rem
Rem: typing.Literal["radrem"] = "radrem"


# Radiation: Exposure (C/kg)
Exposure: typing.Literal["radexpckg"] = "radexpckg"


# Radiation: roentgen (R)
Roentgen: typing.Literal["radr"] = "radr"


# Radiation: Sievert/hour (Sv/h)
SievertPerHour: typing.Literal["radsvh"] = "radsvh"


# Radiation: milliSievert/hour (mSv/h)
MillisievertPerHour: typing.Literal["radmsvh"] = "radmsvh"


# Radiation: microSievert/hour (µSv/h)
MicrosievertPerHour: typing.Literal["radusvh"] = "radusvh"


# Rotational Speed: Revolutions per minute (rpm)
RevolutionsPerMinute: typing.Literal["rotrpm"] = "rotrpm"


# Rotational Speed: Hertz (Hz)
HertzRotation: typing.Literal["rothz"] = "rothz"


# Rotational Speed: Kilohertz (kHz)
KilohertzRotation: typing.Literal["rotkhz"] = "rotkhz"


# Rotational Speed: Megahertz (MHz)
MegahertzRotation: typing.Literal["rotmhz"] = "rotmhz"


# Rotational Speed: Gigahertz (GHz)
GigahertzRotation: typing.Literal["rotghz"] = "rotghz"


# Rotational Speed: Radians per second (rad/s)
RadiansPerSecond: typing.Literal["rotrads"] = "rotrads"


# Rotational Speed: Degrees per second (°/s)
DegreesPerSecond: typing.Literal["rotdegs"] = "rotdegs"


# Temperature: Celsius (°C)
Celsius: typing.Literal["celsius"] = "celsius"


# Temperature: Fahrenheit (°F)
Fahrenheit: typing.Literal["fahrenheit"] = "fahrenheit"


# Temperature: Kelvin (K)
Kelvin: typing.Literal["kelvin"] = "kelvin"


# Time: Hertz (1/s)
Hertz: typing.Literal["hertz"] = "hertz"


# Time: nanoseconds (ns)
Nanoseconds: typing.Literal["ns"] = "ns"


# Time: microseconds (µs)
Microseconds: typing.Literal["µs"] = "µs"


# Time: milliseconds (ms)
Milliseconds: typing.Literal["ms"] = "ms"


# Time: seconds (s)
Seconds: typing.Literal["s"] = "s"


# Time: minutes (m)
Minutes: typing.Literal["m"] = "m"


# Time: hours (h)
Hours: typing.Literal["h"] = "h"


# Time: days (d)
Days: typing.Literal["d"] = "d"


# Time: duration (ms)
DurationMilliseconds: typing.Literal["dtdurationms"] = "dtdurationms"


# Time: duration (s)
DurationSeconds: typing.Literal["dtdurations"] = "dtdurations"


# Time: duration (hh:mm:ss)
DurationInHoursMinutesSeconds: typing.Literal["dthms"] = "dthms"


# Time: duration (d hh:mm:ss)
DurationInDaysHoursMinutesSeconds: typing.Literal["dtdhms"] = "dtdhms"


# Time: Timeticks (s/100)
Timeticks: typing.Literal["timeticks"] = "timeticks"


# Time: clock (ms)
ClockMilliseconds: typing.Literal["clockms"] = "clockms"


# Time: clock (s)
ClockSeconds: typing.Literal["clocks"] = "clocks"


# Throughput: counts/sec (cps)
CountsPerSecond: typing.Literal["cps"] = "cps"


# Throughput: ops/sec (ops)
OpsPerSecond: typing.Literal["ops"] = "ops"


# Throughput: requests/sec (rps)
RequestsPerSecond: typing.Literal["reqps"] = "reqps"


# Throughput: reads/sec (rps)
ReadsPerSecond: typing.Literal["rps"] = "rps"


# Throughput: writes/sec (wps)
WritesPerSecond: typing.Literal["wps"] = "wps"


# Throughput: I/O ops/sec (iops)
IOOpsPerSecond: typing.Literal["iops"] = "iops"


# Throughput: events/sec (eps)
EventsPerSecond: typing.Literal["eps"] = "eps"


# Throughput: messages/sec (mps)
MessagesPerSecond: typing.Literal["mps"] = "mps"


# Throughput: records/sec (rps)
RecordsPerSecond: typing.Literal["recps"] = "recps"


# Throughput: rows/sec (rps)
RowsPerSecond: typing.Literal["rowsps"] = "rowsps"


# Throughput: counts/min (cpm)
CountsPerMinute: typing.Literal["cpm"] = "cpm"


# Throughput: ops/min (opm)
OpsPerMinute: typing.Literal["opm"] = "opm"


# Throughput: requests/min (rpm)
RequestsPerMinute: typing.Literal["reqpm"] = "reqpm"


# Throughput: reads/min (rpm)
ReadsPerMinute: typing.Literal["rpm"] = "rpm"


# Throughput: writes/min (wpm)
WritesPerMinute: typing.Literal["wpm"] = "wpm"


# Throughput: events/min (epm)
EventsPerMinute: typing.Literal["epm"] = "epm"


# Throughput: messages/min (mpm)
MessagesPerMinute: typing.Literal["mpm"] = "mpm"


# Throughput: records/min (rpm)
RecordsPerMinute: typing.Literal["recpm"] = "recpm"


# Throughput: rows/min (rpm)
RowsPerMinute: typing.Literal["rowspm"] = "rowspm"


# Velocity: meters/second (m/s)
MetersPerSecond: typing.Literal["velocityms"] = "velocityms"


# Velocity: kilometers/hour (km/h)
KilometersPerHour: typing.Literal["velocitykmh"] = "velocitykmh"


# Velocity: miles/hour (mph)
MilesPerHour: typing.Literal["velocitymph"] = "velocitymph"


# Velocity: knot (kn)
Knot: typing.Literal["velocityknot"] = "velocityknot"


# Volume: millilitre (mL)
Millilitre: typing.Literal["mlitre"] = "mlitre"


# Volume: litre (L)
Litre: typing.Literal["litre"] = "litre"


# Volume: cubic meter
CubicMeter: typing.Literal["m3"] = "m3"


# Volume: Normal cubic meter
NormalCubicMeter: typing.Literal["Nm3"] = "Nm3"


# Volume: cubic decimeter
CubicDecimeter: typing.Literal["dm3"] = "dm3"


# Volume: gallons
Gallons: typing.Literal["gallons"] = "gallons"


# Boolean: True / False
Bool: typing.Literal["bool"] = "bool"


# Boolean: Yes / No
BoolYesNo: typing.Literal["bool_yes_no"] = "bool_yes_no"


# Boolean: On / Off
BoolOnOff: typing.Literal["bool_on_off"] = "bool_on_off"






# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
from ..cog import runtime as cogruntime


class Options:
    show_threshold_labels: bool
    show_threshold_markers: bool
    sizing: common.BarGaugeSizing
    min_viz_width: int
    reduce_options: common.ReduceDataOptions
    text: typing.Optional[common.VizTextDisplayOptions]
    min_viz_height: int
    orientation: common.VizOrientation

    def __init__(self, show_threshold_labels: bool = False, show_threshold_markers: bool = True, sizing: typing.Optional[common.BarGaugeSizing] = None, min_viz_width: int = 75, reduce_options: typing.Optional[common.ReduceDataOptions] = None, text: typing.Optional[common.VizTextDisplayOptions] = None, min_viz_height: int = 75, orientation: typing.Optional[common.VizOrientation] = None):
        self.show_threshold_labels = show_threshold_labels
        self.show_threshold_markers = show_threshold_markers
        self.sizing = sizing if sizing is not None else common.BarGaugeSizing.AUTO
        self.min_viz_width = min_viz_width
        self.reduce_options = reduce_options if reduce_options is not None else common.ReduceDataOptions()
        self.text = text
        self.min_viz_height = min_viz_height
        self.orientation = orientation if orientation is not None else common.VizOrientation.AUTO

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "showThresholdLabels": self.show_threshold_labels,
            "showThresholdMarkers": self.show_threshold_markers,
            "sizing": self.sizing,
            "minVizWidth": self.min_viz_width,
            "reduceOptions": self.reduce_options,
            "minVizHeight": self.min_viz_height,
            "orientation": self.orientation,
        }
        if self.text is not None:
            payload["text"] = self.text
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "showThresholdLabels" in data:
            args["show_threshold_labels"] = data["showThresholdLabels"]
        if "showThresholdMarkers" in data:
            args["show_threshold_markers"] = data["showThresholdMarkers"]
        if "sizing" in data:
            args["sizing"] = data["sizing"]
        if "minVizWidth" in data:
            args["min_viz_width"] = data["minVizWidth"]
        if "reduceOptions" in data:
            args["reduce_options"] = common.ReduceDataOptions.from_json(data["reduceOptions"])
        if "text" in data:
            args["text"] = common.VizTextDisplayOptions.from_json(data["text"])
        if "minVizHeight" in data:
            args["min_viz_height"] = data["minVizHeight"]
        if "orientation" in data:
            args["orientation"] = data["orientation"]        

        return cls(**args)


def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="gauge",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import variants as cogvariants
from ..models import dashboard
from ..cog import runtime as cogruntime


Expr: typing.TypeAlias = typing.Union['TypeMath', 'TypeReduce', 'TypeResample', 'TypeClassicConditions', 'TypeThreshold', 'TypeSql']


class TypeMath(cogvariants.Dataquery):
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    # General math expression
    expression: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: str
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ExprTypeMathResultAssertions']
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['ExprTypeMathTimeRange']
    type_val: typing.Literal["math"]

    def __init__(self, datasource: typing.Optional[dashboard.DataSourceRef] = None, expression: str = "", hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, query_type: typing.Optional[str] = None, ref_id: str = "", result_assertions: typing.Optional['ExprTypeMathResultAssertions'] = None, time_range: typing.Optional['ExprTypeMathTimeRange'] = None):
        self.datasource = datasource
        self.expression = expression
        self.hide = hide
        self.interval_ms = interval_ms
        self.max_data_points = max_data_points
        self.query_type = query_type
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.time_range = time_range
        self.type_val = "math"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "expression": self.expression,
            "refId": self.ref_id,
            "type": self.type_val,
        }
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "expression" in data:
            args["expression"] = data["expression"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ExprTypeMathResultAssertions.from_json(data["resultAssertions"])
        if "timeRange" in data:
            args["time_range"] = ExprTypeMathTimeRange.from_json(data["timeRange"])        

        return cls(**args)


class TypeReduce(cogvariants.Dataquery):
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    # Reference to single query result
    expression: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # The reducer
    # Possible enum values:
    #  - `"sum"` 
    #  - `"mean"` 
    #  - `"min"` 
    #  - `"max"` 
    #  - `"count"` 
    #  - `"last"` 
    #  - `"median"` 
    reducer: typing.Literal["sum", "mean", "min", "max", "count", "last", "median"]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: str
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ExprTypeReduceResultAssertions']
    # Reducer Options
    settings: typing.Optional['ExprTypeReduceSettings']
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['ExprTypeReduceTimeRange']
    type_val: typing.Literal["reduce"]

    def __init__(self, datasource: typing.Optional[dashboard.DataSourceRef] = None, expression: str = "", hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, query_type: typing.Optional[str] = None, reducer: typing.Optional[typing.Literal["sum", "mean", "min", "max", "count", "last", "median"]] = None, ref_id: str = "", result_assertions: typing.Optional['ExprTypeReduceResultAssertions'] = None, settings: typing.Optional['ExprTypeReduceSettings'] = None, time_range: typing.Optional['ExprTypeReduceTimeRange'] = None):
        self.datasource = datasource
        self.expression = expression
        self.hide = hide
        self.interval_ms = interval_ms
        self.max_data_points = max_data_points
        self.query_type = query_type
        self.reducer = reducer if reducer is not None else "sum"
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.settings = settings
        self.time_range = time_range
        self.type_val = "reduce"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "expression": self.expression,
            "reducer": self.reducer,
            "refId": self.ref_id,
            "type": self.type_val,
        }
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "expression" in data:
            args["expression"] = data["expression"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "reducer" in data:
            args["reducer"] = data["reducer"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ExprTypeReduceResultAssertions.from_json(data["resultAssertions"])
        if "settings" in data:
            args["settings"] = ExprTypeReduceSettings.from_json(data["settings"])
        if "timeRange" in data:
            args["time_range"] = ExprTypeReduceTimeRange.from_json(data["timeRange"])        

        return cls(**args)


class TypeResample(cogvariants.Dataquery):
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    # The downsample function
    # Possible enum values:
    #  - `"sum"` 
    #  - `"mean"` 
    #  - `"min"` 
    #  - `"max"` 
    #  - `"count"` 
    #  - `"last"` 
    #  - `"median"` 
    downsampler: typing.Literal["sum", "mean", "min", "max", "count", "last", "median"]
    # The math expression
    expression: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: str
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ExprTypeResampleResultAssertions']
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['ExprTypeResampleTimeRange']
    type_val: typing.Literal["resample"]
    # The upsample function
    # Possible enum values:
    #  - `"pad"` Use the last seen value
    #  - `"backfilling"` backfill
    #  - `"fillna"` Do not fill values (nill)
    upsampler: typing.Literal["pad", "backfilling", "fillna"]
    # The time duration
    window: str

    def __init__(self, datasource: typing.Optional[dashboard.DataSourceRef] = None, downsampler: typing.Optional[typing.Literal["sum", "mean", "min", "max", "count", "last", "median"]] = None, expression: str = "", hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, query_type: typing.Optional[str] = None, ref_id: str = "", result_assertions: typing.Optional['ExprTypeResampleResultAssertions'] = None, time_range: typing.Optional['ExprTypeResampleTimeRange'] = None, upsampler: typing.Optional[typing.Literal["pad", "backfilling", "fillna"]] = None, window: str = ""):
        self.datasource = datasource
        self.downsampler = downsampler if downsampler is not None else "sum"
        self.expression = expression
        self.hide = hide
        self.interval_ms = interval_ms
        self.max_data_points = max_data_points
        self.query_type = query_type
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.time_range = time_range
        self.type_val = "resample"
        self.upsampler = upsampler if upsampler is not None else "pad"
        self.window = window

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "downsampler": self.downsampler,
            "expression": self.expression,
            "refId": self.ref_id,
            "type": self.type_val,
            "upsampler": self.upsampler,
            "window": self.window,
        }
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "downsampler" in data:
            args["downsampler"] = data["downsampler"]
        if "expression" in data:
            args["expression"] = data["expression"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ExprTypeResampleResultAssertions.from_json(data["resultAssertions"])
        if "timeRange" in data:
            args["time_range"] = ExprTypeResampleTimeRange.from_json(data["timeRange"])
        if "upsampler" in data:
            args["upsampler"] = data["upsampler"]
        if "window" in data:
            args["window"] = data["window"]        

        return cls(**args)


class TypeClassicConditions(cogvariants.Dataquery):
    conditions: list['ExprTypeClassicConditionsConditions']
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: str
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ExprTypeClassicConditionsResultAssertions']
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['ExprTypeClassicConditionsTimeRange']
    type_val: typing.Literal["classic_conditions"]

    def __init__(self, conditions: typing.Optional[list['ExprTypeClassicConditionsConditions']] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, query_type: typing.Optional[str] = None, ref_id: str = "", result_assertions: typing.Optional['ExprTypeClassicConditionsResultAssertions'] = None, time_range: typing.Optional['ExprTypeClassicConditionsTimeRange'] = None):
        self.conditions = conditions if conditions is not None else []
        self.datasource = datasource
        self.hide = hide
        self.interval_ms = interval_ms
        self.max_data_points = max_data_points
        self.query_type = query_type
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.time_range = time_range
        self.type_val = "classic_conditions"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "conditions": self.conditions,
            "refId": self.ref_id,
            "type": self.type_val,
        }
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "conditions" in data:
            args["conditions"] = [ExprTypeClassicConditionsConditions.from_json(item) for item in data["conditions"]]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ExprTypeClassicConditionsResultAssertions.from_json(data["resultAssertions"])
        if "timeRange" in data:
            args["time_range"] = ExprTypeClassicConditionsTimeRange.from_json(data["timeRange"])        

        return cls(**args)


class TypeThreshold(cogvariants.Dataquery):
    # Threshold Conditions
    conditions: list['ExprTypeThresholdConditions']
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    # Reference to single query result
    expression: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: str
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ExprTypeThresholdResultAssertions']
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['ExprTypeThresholdTimeRange']
    type_val: typing.Literal["threshold"]

    def __init__(self, conditions: typing.Optional[list['ExprTypeThresholdConditions']] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, expression: str = "", hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, query_type: typing.Optional[str] = None, ref_id: str = "", result_assertions: typing.Optional['ExprTypeThresholdResultAssertions'] = None, time_range: typing.Optional['ExprTypeThresholdTimeRange'] = None):
        self.conditions = conditions if conditions is not None else []
        self.datasource = datasource
        self.expression = expression
        self.hide = hide
        self.interval_ms = interval_ms
        self.max_data_points = max_data_points
        self.query_type = query_type
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.time_range = time_range
        self.type_val = "threshold"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "conditions": self.conditions,
            "expression": self.expression,
            "refId": self.ref_id,
            "type": self.type_val,
        }
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "conditions" in data:
            args["conditions"] = [ExprTypeThresholdConditions.from_json(item) for item in data["conditions"]]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "expression" in data:
            args["expression"] = data["expression"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ExprTypeThresholdResultAssertions.from_json(data["resultAssertions"])
        if "timeRange" in data:
            args["time_range"] = ExprTypeThresholdTimeRange.from_json(data["timeRange"])        

        return cls(**args)


class TypeSql(cogvariants.Dataquery):
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    expression: str
    format_val: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: str
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ExprTypeSqlResultAssertions']
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['ExprTypeSqlTimeRange']
    type_val: typing.Literal["sql"]

    def __init__(self, datasource: typing.Optional[dashboard.DataSourceRef] = None, expression: str = "", format_val: str = "", hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, query_type: typing.Optional[str] = None, ref_id: str = "", result_assertions: typing.Optional['ExprTypeSqlResultAssertions'] = None, time_range: typing.Optional['ExprTypeSqlTimeRange'] = None):
        self.datasource = datasource
        self.expression = expression
        self.format_val = format_val
        self.hide = hide
        self.interval_ms = interval_ms
        self.max_data_points = max_data_points
        self.query_type = query_type
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.time_range = time_range
        self.type_val = "sql"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "expression": self.expression,
            "format": self.format_val,
            "refId": self.ref_id,
            "type": self.type_val,
        }
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "expression" in data:
            args["expression"] = data["expression"]
        if "format" in data:
            args["format_val"] = data["format"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ExprTypeSqlResultAssertions.from_json(data["resultAssertions"])
        if "timeRange" in data:
            args["time_range"] = ExprTypeSqlTimeRange.from_json(data["timeRange"])        

        return cls(**args)


class TypeMathOrTypeReduceOrTypeResampleOrTypeClassicConditionsOrTypeThresholdOrTypeSql:
    type_math: typing.Optional['TypeMath']
    type_reduce: typing.Optional['TypeReduce']
    type_resample: typing.Optional['TypeResample']
    type_classic_conditions: typing.Optional['TypeClassicConditions']
    type_threshold: typing.Optional['TypeThreshold']
    type_sql: typing.Optional['TypeSql']

    def __init__(self, type_math: typing.Optional['TypeMath'] = None, type_reduce: typing.Optional['TypeReduce'] = None, type_resample: typing.Optional['TypeResample'] = None, type_classic_conditions: typing.Optional['TypeClassicConditions'] = None, type_threshold: typing.Optional['TypeThreshold'] = None, type_sql: typing.Optional['TypeSql'] = None):
        self.type_math = type_math
        self.type_reduce = type_reduce
        self.type_resample = type_resample
        self.type_classic_conditions = type_classic_conditions
        self.type_threshold = type_threshold
        self.type_sql = type_sql

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.type_math is not None:
            payload["TypeMath"] = self.type_math
        if self.type_reduce is not None:
            payload["TypeReduce"] = self.type_reduce
        if self.type_resample is not None:
            payload["TypeResample"] = self.type_resample
        if self.type_classic_conditions is not None:
            payload["TypeClassicConditions"] = self.type_classic_conditions
        if self.type_threshold is not None:
            payload["TypeThreshold"] = self.type_threshold
        if self.type_sql is not None:
            payload["TypeSql"] = self.type_sql
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "TypeMath" in data:
            args["type_math"] = TypeMath.from_json(data["TypeMath"])
        if "TypeReduce" in data:
            args["type_reduce"] = TypeReduce.from_json(data["TypeReduce"])
        if "TypeResample" in data:
            args["type_resample"] = TypeResample.from_json(data["TypeResample"])
        if "TypeClassicConditions" in data:
            args["type_classic_conditions"] = TypeClassicConditions.from_json(data["TypeClassicConditions"])
        if "TypeThreshold" in data:
            args["type_threshold"] = TypeThreshold.from_json(data["TypeThreshold"])
        if "TypeSql" in data:
            args["type_sql"] = TypeSql.from_json(data["TypeSql"])        

        return cls(**args)


class ExprTypeMathResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class ExprTypeMathTimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class ExprTypeReduceResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class ExprTypeReduceSettings:
    # Non-number reduce behavior
    # Possible enum values:
    #  - `"dropNN"` Drop non-numbers
    #  - `"replaceNN"` Replace non-numbers
    mode: typing.Literal["dropNN", "replaceNN"]
    # Only valid when mode is replace
    replace_with_value: typing.Optional[float]

    def __init__(self, mode: typing.Optional[typing.Literal["dropNN", "replaceNN"]] = None, replace_with_value: typing.Optional[float] = None):
        self.mode = mode if mode is not None else "dropNN"
        self.replace_with_value = replace_with_value

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "mode": self.mode,
        }
        if self.replace_with_value is not None:
            payload["replaceWithValue"] = self.replace_with_value
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]
        if "replaceWithValue" in data:
            args["replace_with_value"] = data["replaceWithValue"]        

        return cls(**args)


class ExprTypeReduceTimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class ExprTypeResampleResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class ExprTypeResampleTimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class ExprTypeClassicConditionsConditionsEvaluator:
    params: list[float]
    # e.g. "gt"
    type_val: str

    def __init__(self, params: typing.Optional[list[float]] = None, type_val: str = ""):
        self.params = params if params is not None else []
        self.type_val = type_val

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "params": self.params,
            "type": self.type_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "params" in data:
            args["params"] = data["params"]
        if "type" in data:
            args["type_val"] = data["type"]        

        return cls(**args)


class ExprTypeClassicConditionsConditionsOperator:
    type_val: typing.Literal["and", "or", "logic-or"]

    def __init__(self, type_val: typing.Optional[typing.Literal["and", "or", "logic-or"]] = None):
        self.type_val = type_val if type_val is not None else "and"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]        

        return cls(**args)


class ExprTypeClassicConditionsConditionsQuery:
    params: list[str]

    def __init__(self, params: typing.Optional[list[str]] = None):
        self.params = params if params is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "params": self.params,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "params" in data:
            args["params"] = data["params"]        

        return cls(**args)


class ExprTypeClassicConditionsConditionsReducer:
    type_val: str

    def __init__(self, type_val: str = ""):
        self.type_val = type_val

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]        

        return cls(**args)


class ExprTypeClassicConditionsConditions:
    evaluator: 'ExprTypeClassicConditionsConditionsEvaluator'
    operator: 'ExprTypeClassicConditionsConditionsOperator'
    query: 'ExprTypeClassicConditionsConditionsQuery'
    reducer: 'ExprTypeClassicConditionsConditionsReducer'

    def __init__(self, evaluator: typing.Optional['ExprTypeClassicConditionsConditionsEvaluator'] = None, operator: typing.Optional['ExprTypeClassicConditionsConditionsOperator'] = None, query: typing.Optional['ExprTypeClassicConditionsConditionsQuery'] = None, reducer: typing.Optional['ExprTypeClassicConditionsConditionsReducer'] = None):
        self.evaluator = evaluator if evaluator is not None else ExprTypeClassicConditionsConditionsEvaluator()
        self.operator = operator if operator is not None else ExprTypeClassicConditionsConditionsOperator()
        self.query = query if query is not None else ExprTypeClassicConditionsConditionsQuery()
        self.reducer = reducer if reducer is not None else ExprTypeClassicConditionsConditionsReducer()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "evaluator": self.evaluator,
            "operator": self.operator,
            "query": self.query,
            "reducer": self.reducer,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "evaluator" in data:
            args["evaluator"] = ExprTypeClassicConditionsConditionsEvaluator.from_json(data["evaluator"])
        if "operator" in data:
            args["operator"] = ExprTypeClassicConditionsConditionsOperator.from_json(data["operator"])
        if "query" in data:
            args["query"] = ExprTypeClassicConditionsConditionsQuery.from_json(data["query"])
        if "reducer" in data:
            args["reducer"] = ExprTypeClassicConditionsConditionsReducer.from_json(data["reducer"])        

        return cls(**args)


class ExprTypeClassicConditionsResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class ExprTypeClassicConditionsTimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class ExprTypeThresholdConditionsEvaluator:
    params: list[float]
    # e.g. "gt"
    type_val: typing.Literal["gt", "lt", "eq", "ne", "gte", "lte", "within_range", "outside_range", "within_range_included", "outside_range_included"]

    def __init__(self, params: typing.Optional[list[float]] = None, type_val: typing.Optional[typing.Literal["gt", "lt", "eq", "ne", "gte", "lte", "within_range", "outside_range", "within_range_included", "outside_range_included"]] = None):
        self.params = params if params is not None else []
        self.type_val = type_val if type_val is not None else "gt"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "params": self.params,
            "type": self.type_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "params" in data:
            args["params"] = data["params"]
        if "type" in data:
            args["type_val"] = data["type"]        

        return cls(**args)


class ExprTypeThresholdConditionsUnloadEvaluator:
    params: list[float]
    # e.g. "gt"
    type_val: typing.Literal["gt", "lt", "eq", "ne", "gte", "lte", "within_range", "outside_range", "within_range_included", "outside_range_included"]

    def __init__(self, params: typing.Optional[list[float]] = None, type_val: typing.Optional[typing.Literal["gt", "lt", "eq", "ne", "gte", "lte", "within_range", "outside_range", "within_range_included", "outside_range_included"]] = None):
        self.params = params if params is not None else []
        self.type_val = type_val if type_val is not None else "gt"

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "params": self.params,
            "type": self.type_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "params" in data:
            args["params"] = data["params"]
        if "type" in data:
            args["type_val"] = data["type"]        

        return cls(**args)


class ExprTypeThresholdConditions:
    evaluator: 'ExprTypeThresholdConditionsEvaluator'
    loaded_dimensions: typing.Optional[object]
    unload_evaluator: typing.Optional['ExprTypeThresholdConditionsUnloadEvaluator']

    def __init__(self, evaluator: typing.Optional['ExprTypeThresholdConditionsEvaluator'] = None, loaded_dimensions: typing.Optional[object] = None, unload_evaluator: typing.Optional['ExprTypeThresholdConditionsUnloadEvaluator'] = None):
        self.evaluator = evaluator if evaluator is not None else ExprTypeThresholdConditionsEvaluator()
        self.loaded_dimensions = loaded_dimensions
        self.unload_evaluator = unload_evaluator

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "evaluator": self.evaluator,
        }
        if self.loaded_dimensions is not None:
            payload["loadedDimensions"] = self.loaded_dimensions
        if self.unload_evaluator is not None:
            payload["unloadEvaluator"] = self.unload_evaluator
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "evaluator" in data:
            args["evaluator"] = ExprTypeThresholdConditionsEvaluator.from_json(data["evaluator"])
        if "loadedDimensions" in data:
            args["loaded_dimensions"] = data["loadedDimensions"]
        if "unloadEvaluator" in data:
            args["unload_evaluator"] = ExprTypeThresholdConditionsUnloadEvaluator.from_json(data["unloadEvaluator"])        

        return cls(**args)


class ExprTypeThresholdResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class ExprTypeThresholdTimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class ExprTypeSqlResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class ExprTypeSqlTimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    decoding_map_entrypoint_ref_union: dict[str, typing.Union[typing.Type[TypeClassicConditions], typing.Type[TypeMath], typing.Type[TypeReduce], typing.Type[TypeResample], typing.Type[TypeSql], typing.Type[TypeThreshold]]] = {"classic_conditions": TypeClassicConditions, "math": TypeMath, "reduce": TypeReduce, "resample": TypeResample, "sql": TypeSql, "threshold": TypeThreshold}
    return cogruntime.DataqueryConfig(
        identifier="__expr__",
        from_json_hook=lambda data: decoding_map_entrypoint_ref_union[data["type"]].from_json(data),
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


class AccessPolicy:
    # The scope where these policies should apply
    scope: 'ResourceRef'
    # The role that must apply this policy
    role: 'RoleRef'
    # The set of rules to apply.  Note that * is required to modify
    # access policy rules, and that "none" will reject all actions
    rules: list['AccessRule']

    def __init__(self, scope: typing.Optional['ResourceRef'] = None, role: typing.Optional['RoleRef'] = None, rules: typing.Optional[list['AccessRule']] = None):
        self.scope = scope if scope is not None else ResourceRef()
        self.role = role if role is not None else RoleRef()
        self.rules = rules if rules is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "scope": self.scope,
            "role": self.role,
            "rules": self.rules,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "scope" in data:
            args["scope"] = ResourceRef.from_json(data["scope"])
        if "role" in data:
            args["role"] = RoleRef.from_json(data["role"])
        if "rules" in data:
            args["rules"] = [AccessRule.from_json(item) for item in data["rules"]]        

        return cls(**args)


class ResourceRef:
    kind: str
    name: str

    def __init__(self, kind: str = "", name: str = ""):
        self.kind = kind
        self.name = name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "name": self.name,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "kind" in data:
            args["kind"] = data["kind"]
        if "name" in data:
            args["name"] = data["name"]        

        return cls(**args)


class RoleRef:
    # Policies can apply to roles, teams, or users
    # Applying policies to individual users is supported, but discouraged
    kind: typing.Literal["Role", "BuiltinRole", "Team", "User"]
    name: str
    xname: str

    def __init__(self, kind: typing.Optional[typing.Literal["Role", "BuiltinRole", "Team", "User"]] = None, name: str = "", xname: str = ""):
        self.kind = kind if kind is not None else "Role"
        self.name = name
        self.xname = xname

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "name": self.name,
            "xname": self.xname,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "kind" in data:
            args["kind"] = data["kind"]
        if "name" in data:
            args["name"] = data["name"]
        if "xname" in data:
            args["xname"] = data["xname"]        

        return cls(**args)


class AccessRule:
    # The kind this rule applies to (dashboards, alert, etc)
    kind: str
    # READ, WRITE, CREATE, DELETE, ...
    # should move to k8s style verbs like: "get", "list", "watch", "create", "update", "patch", "delete"
    verb: typing.Union[typing.Literal["*"], typing.Literal["none"], str]
    # Specific sub-elements like "alert.rules" or "dashboard.permissions"????
    target: typing.Optional[str]

    def __init__(self, kind: str = "*", verb: typing.Optional[typing.Union[typing.Literal["*"], typing.Literal["none"], str]] = None, target: typing.Optional[str] = None):
        self.kind = kind
        self.verb = verb if verb is not None else "*"
        self.target = target

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "kind": self.kind,
            "verb": self.verb,
        }
        if self.target is not None:
            payload["target"] = self.target
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "kind" in data:
            args["kind"] = data["kind"]
        if "verb" in data:
            args["verb"] = data["verb"]
        if "target" in data:
            args["target"] = data["target"]        

        return cls(**args)






# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..cog import variants as cogvariants
import typing
from ..models import dashboard
import enum
from ..cog import runtime as cogruntime


class Dataquery(cogvariants.Dataquery):
    """
    Manually converted from https://github.com/grafana/google-bigquery-datasource/blob/18680e42ba557791d109c7c540c2c3f2647592f0/src/types.ts#L75
    """

    dataset: typing.Optional[str]
    table: typing.Optional[str]
    project: typing.Optional[str]
    format_val: 'QueryFormat'
    raw_query: typing.Optional[bool]
    raw_sql: str
    location: typing.Optional[str]
    partitioned: typing.Optional[bool]
    partitioned_field: typing.Optional[str]
    convert_to_utc: typing.Optional[bool]
    sharded: typing.Optional[bool]
    query_priority: typing.Optional['QueryPriority']
    time_shift: typing.Optional[str]
    editor_mode: typing.Optional['EditorMode']
    sql: typing.Optional['SQLExpression']
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]

    def __init__(self, dataset: typing.Optional[str] = None, table: typing.Optional[str] = None, project: typing.Optional[str] = None, format_val: typing.Optional['QueryFormat'] = None, raw_query: typing.Optional[bool] = None, raw_sql: str = "", location: typing.Optional[str] = None, partitioned: typing.Optional[bool] = None, partitioned_field: typing.Optional[str] = None, convert_to_utc: typing.Optional[bool] = None, sharded: typing.Optional[bool] = None, query_priority: typing.Optional['QueryPriority'] = None, time_shift: typing.Optional[str] = None, editor_mode: typing.Optional['EditorMode'] = None, sql: typing.Optional['SQLExpression'] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None):
        self.dataset = dataset
        self.table = table
        self.project = project
        self.format_val = format_val if format_val is not None else QueryFormat.TIMESERIES
        self.raw_query = raw_query
        self.raw_sql = raw_sql
        self.location = location
        self.partitioned = partitioned
        self.partitioned_field = partitioned_field
        self.convert_to_utc = convert_to_utc
        self.sharded = sharded
        self.query_priority = query_priority
        self.time_shift = time_shift
        self.editor_mode = editor_mode
        self.sql = sql
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "format": self.format_val,
            "rawSql": self.raw_sql,
            "refId": self.ref_id,
        }
        if self.dataset is not None:
            payload["dataset"] = self.dataset
        if self.table is not None:
            payload["table"] = self.table
        if self.project is not None:
            payload["project"] = self.project
        if self.raw_query is not None:
            payload["rawQuery"] = self.raw_query
        if self.location is not None:
            payload["location"] = self.location
        if self.partitioned is not None:
            payload["partitioned"] = self.partitioned
        if self.partitioned_field is not None:
            payload["partitionedField"] = self.partitioned_field
        if self.convert_to_utc is not None:
            payload["convertToUTC"] = self.convert_to_utc
        if self.sharded is not None:
            payload["sharded"] = self.sharded
        if self.query_priority is not None:
            payload["queryPriority"] = self.query_priority
        if self.time_shift is not None:
            payload["timeShift"] = self.time_shift
        if self.editor_mode is not None:
            payload["editorMode"] = self.editor_mode
        if self.sql is not None:
            payload["sql"] = self.sql
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "dataset" in data:
            args["dataset"] = data["dataset"]
        if "table" in data:
            args["table"] = data["table"]
        if "project" in data:
            args["project"] = data["project"]
        if "format" in data:
            args["format_val"] = data["format"]
        if "rawQuery" in data:
            args["raw_query"] = data["rawQuery"]
        if "rawSql" in data:
            args["raw_sql"] = data["rawSql"]
        if "location" in data:
            args["location"] = data["location"]
        if "partitioned" in data:
            args["partitioned"] = data["partitioned"]
        if "partitionedField" in data:
            args["partitioned_field"] = data["partitionedField"]
        if "convertToUTC" in data:
            args["convert_to_utc"] = data["convertToUTC"]
        if "sharded" in data:
            args["sharded"] = data["sharded"]
        if "queryPriority" in data:
            args["query_priority"] = data["queryPriority"]
        if "timeShift" in data:
            args["time_shift"] = data["timeShift"]
        if "editorMode" in data:
            args["editor_mode"] = data["editorMode"]
        if "sql" in data:
            args["sql"] = SQLExpression.from_json(data["sql"])
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])        

        return cls(**args)


class QueryFormat(enum.IntEnum):
    TIMESERIES = 0
    TABLE = 1


class QueryPriority(enum.StrEnum):
    INTERACTIVE = "INTERACTIVE"
    BATCH = "BATCH"


class EditorMode(enum.StrEnum):
    CODE = "code"
    BUILDER = "builder"


class SQLExpression:
    columns: typing.Optional[list['QueryEditorFunctionExpression']]
    from_val: typing.Optional[str]
    # whereJsonTree?: _
    where_string: typing.Optional[str]
    group_by: typing.Optional[list['QueryEditorGroupByExpression']]
    order_by: typing.Optional['QueryEditorPropertyExpression']
    order_by_direction: typing.Optional['OrderByDirection']
    limit: typing.Optional[int]
    offset: typing.Optional[int]

    def __init__(self, columns: typing.Optional[list['QueryEditorFunctionExpression']] = None, from_val: typing.Optional[str] = None, where_string: typing.Optional[str] = None, group_by: typing.Optional[list['QueryEditorGroupByExpression']] = None, order_by: typing.Optional['QueryEditorPropertyExpression'] = None, order_by_direction: typing.Optional['OrderByDirection'] = None, limit: typing.Optional[int] = None, offset: typing.Optional[int] = None):
        self.columns = columns
        self.from_val = from_val
        self.where_string = where_string
        self.group_by = group_by
        self.order_by = order_by
        self.order_by_direction = order_by_direction
        self.limit = limit
        self.offset = offset

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.columns is not None:
            payload["columns"] = self.columns
        if self.from_val is not None:
            payload["from"] = self.from_val
        if self.where_string is not None:
            payload["whereString"] = self.where_string
        if self.group_by is not None:
            payload["groupBy"] = self.group_by
        if self.order_by is not None:
            payload["orderBy"] = self.order_by
        if self.order_by_direction is not None:
            payload["orderByDirection"] = self.order_by_direction
        if self.limit is not None:
            payload["limit"] = self.limit
        if self.offset is not None:
            payload["offset"] = self.offset
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "columns" in data:
            args["columns"] = [QueryEditorFunctionExpression.from_json(item) for item in data["columns"]]
        if "from" in data:
            args["from_val"] = data["from"]
        if "whereString" in data:
            args["where_string"] = data["whereString"]
        if "groupBy" in data:
            args["group_by"] = [QueryEditorGroupByExpression.from_json(item) for item in data["groupBy"]]
        if "orderBy" in data:
            args["order_by"] = QueryEditorPropertyExpression.from_json(data["orderBy"])
        if "orderByDirection" in data:
            args["order_by_direction"] = data["orderByDirection"]
        if "limit" in data:
            args["limit"] = data["limit"]
        if "offset" in data:
            args["offset"] = data["offset"]        

        return cls(**args)


class QueryEditorFunctionExpression:
    type_val: str
    name: typing.Optional[str]
    parameters: typing.Optional[list['QueryEditorFunctionParameterExpression']]

    def __init__(self, name: typing.Optional[str] = None, parameters: typing.Optional[list['QueryEditorFunctionParameterExpression']] = None):
        self.type_val = QueryEditorExpressionType.FUNCTION
        self.name = name
        self.parameters = parameters

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.name is not None:
            payload["name"] = self.name
        if self.parameters is not None:
            payload["parameters"] = self.parameters
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "parameters" in data:
            args["parameters"] = [QueryEditorFunctionParameterExpression.from_json(item) for item in data["parameters"]]        

        return cls(**args)


class QueryEditorExpressionType(enum.StrEnum):
    PROPERTY = "property"
    OPERATOR = "operator"
    OR = "or"
    AND = "and"
    GROUP_BY = "groupBy"
    FUNCTION = "function"
    FUNCTION_PARAMETER = "functionParameter"


class QueryEditorFunctionParameterExpression:
    type_val: str
    name: typing.Optional[str]

    def __init__(self, name: typing.Optional[str] = None):
        self.type_val = QueryEditorExpressionType.FUNCTION_PARAMETER
        self.name = name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.name is not None:
            payload["name"] = self.name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]        

        return cls(**args)


class QueryEditorGroupByExpression:
    type_val: str
    property_val: 'QueryEditorProperty'

    def __init__(self, property_val: typing.Optional['QueryEditorProperty'] = None):
        self.type_val = QueryEditorExpressionType.GROUP_BY
        self.property_val = property_val if property_val is not None else QueryEditorProperty()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "property": self.property_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "property" in data:
            args["property_val"] = QueryEditorProperty.from_json(data["property"])        

        return cls(**args)


class QueryEditorProperty:
    type_val: 'QueryEditorPropertyType'
    name: typing.Optional[str]

    def __init__(self, type_val: typing.Optional['QueryEditorPropertyType'] = None, name: typing.Optional[str] = None):
        self.type_val = type_val if type_val is not None else QueryEditorPropertyType.STRING
        self.name = name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.name is not None:
            payload["name"] = self.name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "name" in data:
            args["name"] = data["name"]        

        return cls(**args)


class QueryEditorPropertyType(enum.StrEnum):
    STRING = "string"


class QueryEditorPropertyExpression:
    type_val: str
    property_val: 'QueryEditorProperty'

    def __init__(self, property_val: typing.Optional['QueryEditorProperty'] = None):
        self.type_val = QueryEditorExpressionType.PROPERTY
        self.property_val = property_val if property_val is not None else QueryEditorProperty()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "property": self.property_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "property" in data:
            args["property_val"] = QueryEditorProperty.from_json(data["property"])        

        return cls(**args)


class OrderByDirection(enum.StrEnum):
    ASC = "ASC"
    DESC = "DESC"





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="grafana-bigquery-datasource",
        from_json_hook=Dataquery.from_json,
    )

